#ifndef __IUP_IHANDLE_BI__
#define __IUP_IHANDLE_BI__

#include once "iup_table.bi"

' IHandle ptr is more than as an ANY ptr now

' Known native types.
enum InativeType
  IUP_TYPEVOID,     ' No native representation - HBOX, VBOX, ZBOX, FILL, RADIO (handle=&HFFFFFFFF -1 always)
  IUP_TYPECONTROL,  ' Native controls - BUTTON, LABEL, TOGGLE, LIST, TEXT, MULTILINE, FRAME, others
  IUP_TYPECANVAS,   ' Drawing canvas, also used as a base control for custom controls. 
  IUP_TYPEDIALOG,   ' DIALOG
  IUP_TYPEIMAGE,    ' IMAGE
  IUP_TYPEMENU      ' MENU, SUBMENU, ITEM, SEPARATOR
end enum

' Possible number of children.
enum IchildType
  IUP_CHILDNONE, ' can not add children using Append/Insert
  IUP_CHILDMANY  ' can add any number of children IUP_CHILDMANY+n can add n children.
end enum

' General flags.
enum Iflags
  IUP_FLOATING         = &H01, ' is a floating element. FLOATING=Yes
  IUP_FLOATING_IGNORE  = &H02, ' is a floating element. FLOATING=Ignore. Do not compute layout.
  IUP_MAXSIZE          = &H04, ' has the MAXSIZE attribute set
  IUP_MINSIZE          = &H08, ' has the MINSIZE attribute set
  IUP_INTERNAL         = &H10  ' it is an internal element of the container
end enum

' Attribute flags.
' Used by iupClassRegisterAttribute.
enum IattribFlags
  IUPAF_DEFAULT      =   0, ' inheritable, can has a default value, is a string, 
                            ' can call the set/get functions only if mapped, no ID
  IUPAF_NO_INHERIT   =   1, ' is not inheritable
  IUPAF_NO_DEFAULTVALUE= 2, ' has no default value
  IUPAF_NO_STRING    =   4, ' is not a string
  IUPAF_NOT_MAPPED   =   8, ' will call the set/get functions also when not mapped
  IUPAF_HAS_ID       =  16, ' can has an ID at the end of the name, automatically set by iupClassRegisterAttributeId */
  IUPAF_READONLY     =  32, ' is read-only, can not be changed
  IUPAF_WRITEONLY    =  64, ' is write-only, usually an action
  IUPAF_HAS_ID2      = 128, ' can has two IDs at the end of the name, automatically set by iupClassRegisterAttributeId2
  IUPAF_CALLBACK     = 256, ' is a callback, not an attribute
  IUPAF_NO_SAVE      = 512, ' can NOT be directly saved, should have at least manual processing
  IUPAF_NOT_SUPPORTED=1024, ' not supported in that driver
  IUPAF_IHANDLENAME  =2048  ' is an Ihandle* name, associated with IupSetHandle
end enum


type IHandle as IHandle_

type IClass as Iclass_

type Iclass_
  ' configuration parameters.
  as zstring ptr name     ' class name. No default, must be initialized.
  as zstring ptr format   ' Creation parameters format of the class.
                          ' Used only for LED parsing.
                          ' It can have none (NULL), one or more of the following.
                          ' "b" = (unsigned char) - byte
                          ' "c" = (unsigned char*) - array of byte
                          ' "i" = (int) - integer
                          ' "j" = (int*) - array of integer
                          ' "f" = (float) - real
                          ' "s" = (char*) - string 
                          ' "a" = (char*) - name of the ACTION callback
                          ' "h" = (Ihandle*) - element handle
                          ' "g" = (Ihandle**) - array of element handle
                   
  as InativeType nativetype ' native type. Default is IUP_TYPEVOID.
  as IchildType  childtype  ' children count enum: none, many, or n, as described in \ref IchildType. Default is IUP_CHILDNONE.
                            ' This identifies a container that can be manipulated with IupReparent, IupAppend and IupInsert.
                            ' Used to control the allowed number of children and define its behavior in the layout processing.
                            ' The element can still have hidden children even if this is none.
  as integer is_interactive ' keyboard interactive boolean, 
                            ' true if the class can have the keyboard input focus. Default is false.
  as integer has_attrib_id  ' indicate if any attribute is numbered. Default is not. Can be 1 or 2.

  as Iclass_ ptr parent     ' class parent to implement inheritance.
                            ' Class name must be different.
                            ' Creation parameters should be the same or repace the parents creation function.
                            ' Native type should be the same.
                            ' Child type should be a more restrictive or equal type (many->one->none).
                            ' Attribute functions will have only one common table.
                            ' All methods can be changed, set to NULL, switched, etc.

  as Itable ptr attrib_func ' table of functions to handle attributes, only one per class tree

  ' Class methods.

  ' Method that allocates a new instance of the class.
  ' Used by inherited classes in \ref iupClassNew.
  new_ as function cdecl () as Iclass ptr

  ' Method that release the memory allocated by the class.
  ' Called only once at \ref iupClassRelease.
  release as sub cdecl (ic as Iclass ptr)

  ' Method that creates the element and process the creation parameters.
  ' Called only from IupCreate.
  ' The parameters can be NULL for all the controls.
  ' The control should also depend on attributes set before IupMap.
  ' Must return IUP_NOERROR or IUP_ERROR.
  ' Can be NULL, like all methods.
  create as function cdecl (ih as Ihandle ptr, params as any ptr) as integer

  ' Method that map (create) the control to the native system.
  ' Called only from IupMap.
  ' Must return IUP_NOERROR or IUP_ERROR.
  map as function cdecl (ih as Ihandle ptr) as integer

  ' Method that unmap (destroy) the control from the native system.
  ' Called only from IupUnmap if the control is mapped.
  unmap as sub cdecl (ih as ihandle ptr)

  ' Method that destroys the element. \n
  ' Called only from IupDestroy. Always called even if the control is not mapped.
  destroy as sub cdecl (ih as Ihandle ptr)

  ' Returns the actual parent to add a child. The default implementation returns itself.
  ' Called only from IupAppend or IupReparent.
  ' This allows IUP elements to be a combination of other IUP elements in a single IUP element.
  GetInnerContainer as function cdecl (ih as Ihandle ptr) as Ihandle ptr

  ' Returns the internal native parent. The default implementation returns the handle of itself
  ' Called from \ref iupChildTreeGetNativeParentHandle.
  ' This allows native elements to have an internal container
  ' that will be the actual native parent, or in other words allows native elements to be a combination of 
  ' other native elements in a single IUP element. 
  ' The actual native parent may depend on the child tree (see IupTabs for an example).
  GetInnerNativeContainerHandle as function cdecl (ih as Ihandle ptr, child as Ihandle ptr) as any ptr

  ' Notifies the element that a child was appended using IupAppend.
  ' Called only from IupAppend or IupReparent. The child is not mapped, but the parent can be mapped.
  ChildAdded as sub cdecl (ih as Ihandle ptr, child as Ihandle ptr)

  ' Notifies the element that a child was removed using IupDetach.
  ' Called only from IupDetach or IupReparent. The child is not mapped, but the parent can be mapped.
  ChildRemoved as sub cdecl (ih as Ihandle ptr, child as Ihandle ptr)

  ' Method that update size and position of the native control.
  ' Called only from iupLayoutUpdate and if the element is mapped.
  LayoutUpdate as sub cdecl (ih as Ihandle ptr)

  ' Method that computes the natural size based on the user size and the actual natural size.
  ' Should update expand if a container, but does NOT depends on expand to compute the natural size.
  ' Must call the iupBaseComputeNaturalSize for each children.
  ' First calculate the native size for the children, then for the element.
  ' Also called before the element is mapped, so it must be independent of the native control.
  ' First call done at iupLayoutCompute for the dialog.
  ComputeNaturalSize as sub cdecl (ih as Ihandle ptr, w as integer ptr, h as integer ptr, children_expand as integer ptr)

  ' Method that calculates and updates the current size of children based on the available size,
  ' the natural size and the expand configuration.
  ' Called only if there is any children.
  ' Must call \ref iupBaseSetCurrentSize for each children. 
  ' shrink is the dialog attribute passed here for optimization.
  ' Also called before the element is mapped, so it must be independent of the native control.
  ' First call done at iupLayoutCompute for the dialog.
  SetChildrenCurrentSize as sub cdecl (ih as Ihandle ptr, shrink as integer)

  ' Method that calculates and updates the position relative to the parent.
  ' Called only if there is any children.
  ' Must call iupBaseSetPosition for each children.
  ' Also called before the element is mapped, so it must be independent of the native control.
  ' First call done at iupLayoutCompute for the dialog.
  SetChildrenPosition as sub cdecl (ih as Ihandle ptr, x as integer, y as integer)

  ' Method that shows a popup dialog. Called only for native pre-defined dialogs.
  ' The element is not mapped.
  ' Must return IUP_ERROR or IUP_NOERROR.
  ' Called only from iupDialogPopup.
   DlgPopup as function cdecl (ih as Ihandle ptr, x as integer, y as integer) as integer
end type

type _InativeHandle as any : type InativeHandle as _InativeHandle ptr
type _IcontrolData  as any : type IcontrolData as _IcontrolData
type Ihandle_
  as zstring * 4 sig      ' IUP Signature, initialized with "IUP", cleared on destroy
  as Iclass_ ptr iclass   ' Ihandle Class
  as Itable ptr attrib    ' attributes table
  as integer serial       ' serial number used for controls that need a numeric id, initialized with -1
  as InativeHandle handle ' native handle. initialized when mapped. InativeHandle definition is system dependent.
  as integer expand       ' expand configuration, a combination of Iexpand, for containers is a combination of the children expand's
  as Iflags flags          ' flags configuration, a combination of Iflags
  as integer x, y         ' upper-left corner relative to the native parent. always 0 for the dialog.
  as integer userwidth    ' user defined size for the control using SIZE or RASTERSIZE   
  as integer userheight 
  as integer naturalwidth ' the calculated size based in the control contents and the user size
  as integer naturalheight 
  as integer currentwidth ' actual size of the control in pixels (window size, including decorations and margins).
  as integer currentheight
  as Ihandle_ ptr parent     ' previous control in the hierarchy tree
  as Ihandle_ ptr firstchild ' first child control in the hierarchy tree
  as Ihandle_ ptr brother    ' next control inside parent
  as IcontrolData ptr data   ' private control data. automatically freed if not NULL in destroy
end type

#endif
